from django.contrib import admin

from relatedwidget import RelatedWidgetWrapperBase

from freeplay.models import Region, Bit, ContentBit
from freeplay.models import Item, Template


class RegionOptions(RelatedWidgetWrapperBase, admin.ModelAdmin):

    list_display = ("name", "slug", "min_items", "max_items")


class BitOptions(admin.ModelAdmin):

    list_display = ("template", "kind", "name", "context_name")


class BitInline(admin.StackedInline):
    model = Bit


class TemplateOptions(admin.ModelAdmin):
    list_display = ("name", "active_item_count", "bit_names")

    inlines = [
        BitInline,
    ]

    def active_item_count(self, obj):
        return sum([r.items.count() for r in obj.region_set.all()])

    def bit_names(self, obj):
        return ", ".join([b.name for b in obj.bits.all()])


class ItemOptions(RelatedWidgetWrapperBase, admin.ModelAdmin):

    list_display = ("label", "region", "order")
    list_filter = ("region", )


admin.site.register(Region, RegionOptions)
admin.site.register(Template, TemplateOptions)
admin.site.register(Bit, BitOptions)
admin.site.register(Item, ItemOptions)
admin.site.register(ContentBit)



from django import template

from freeplay.models import Region, Item

register = template.Library()


@register.assignment_tag
def content_bits(region_slug):
    try:
        region = Region.objects.get(slug=region_slug)
        return region.items.all()
    except:
        return None


@register.assignment_tag
def get_bit(region_slug, item_slug=None):
    if item_slug:
        try:
            return Item.objects.get(
                region__slug=region_slug,
                slug=item_slug
            )
        except Item.DoesNotExist:
            return None
    else:
        try:
            return Item.objects.filter(
                region__slug=region_slug
            )[0]
        except IndexError:
            return None



from django.shortcuts import get_object_or_404, redirect
from django.template.response import TemplateResponse

from django.contrib import messages
from django.contrib.auth.decorators import permission_required

from .forms import ItemForm
from .models import Region
from .settings import FREEPLAY


class FreeplayResponse(TemplateResponse):

    def __init__(self, request, template, context, **kwargs):

        context.update(FREEPLAY)

        super(FreeplayResponse, self).__init__(
            request, template, context=context, **kwargs)


@permission_required("freeplay.add_contentbit", raise_exception=True)
def dashboard(request):

    regions = Region.objects.all()

    return FreeplayResponse(request,
        "admin/freeplay/dashboard.html",
        {
        "regions": regions
        }
    )


@permission_required("freeplay.add_contentbit", raise_exception=True)
def create_item(request, region_pk):
    region = get_object_or_404(Region, pk=region_pk)

    form = ItemForm(region=region)

    if request.method == "POST":
        form = ItemForm(request.POST, request.FILES, region=region)
        if form.is_valid():
            form.save()
            messages.success(request, "The item was added successfully")
            return redirect("freeplay_dashboard")

    return FreeplayResponse(request,
        "admin/freeplay/{0}item_form.html".format(
            "_" if request.is_ajax() else ""),
        {
            "region": region,
            "form": form,
            "add": True,
            "opts": {
                "verbose_name": "Item for %s" % region.name
            }
        }
    )


@permission_required("freeplay.change_contentbit", raise_exception=True)
def change_item(request, region_pk, item_pk):
    region = get_object_or_404(Region, pk=region_pk)
    item = get_object_or_404(region.items, pk=item_pk)

    form = ItemForm(region=region, instance=item)

    if request.method == "POST":
        form = ItemForm(request.POST, request.FILES, region=region, instance=item)
        if form.is_valid():
            form.save()
            messages.success(request, "The item was updated successfully")
            return redirect("freeplay_dashboard")

    return FreeplayResponse(request,
        "admin/freeplay/{0}item_form.html".format(
            "_" if request.is_ajax() else ""),
        {
            "region": region,
            "form": form,
            "add": False,
            "instance": item,
            "opts": {
                "verbose_name": "Item for %s" % region.name
            }
        }
    )


@permission_required("freeplay.delete_contentbit", raise_exception=True)
def delete_item(request, region_pk, item_pk):
    region = get_object_or_404(Region, pk=region_pk)
    item = get_object_or_404(region.items, pk=item_pk)

    form = ItemForm(region=region, instance=item)

    if request.method == "POST":
        item.delete()
        messages.success(request, "The item was deleted successfully")
        return redirect("freeplay_dashboard")

    return FreeplayResponse(request,
        "admin/freeplay/{0}item_delete.html".format(
            "_" if request.is_ajax() else ""),
        {
            "region": region,
            "instance": item,
            "opts": {
                "verbose_name": "Item for %s" % region.name
            }
        }
    )



from django.conf import settings


FREEPLAY = {
    "CHOSEN_PATH": 'chosen_v1.0.0/chosen.jquery.min.js',
    "SLUGIFY_PATH": 'jquery-slugify/dist/slugify.min.js',
    "MASONRY_PATH": 'masonry/jquery.masonry.min.js'
}

if hasattr(settings, "FREEPLAY"):
    FREEPLAY.update(settings.FREEPLAY)



from django.conf.urls import patterns, url


urlpatterns = patterns("freeplay.views",
    url(r"^$", "dashboard", {}, name="freeplay_dashboard"),
    url(r"^(?P<region_pk>\d+?)/item/add/$",
        "create_item",
        name="freeplay_item_add"),
    url(r"^(?P<region_pk>\d+?)/item/(?P<item_pk>\d+?)/$",
        "change_item",
        name="freeplay_item_change"),
    url(r"^(?P<region_pk>\d+?)/item/(?P<item_pk>\d+?)/delete/$",
        "delete_item",
        name="freeplay_item_delete"),
)



from setuptools import setup, find_packages


setup(
    name = "django-freeplay",
    version = "0.1.8",
    author = "Dave Lowe",
    author_email = "dave@hellopullswitch.com",
    description = "Easily add CMS functionality to your Django site",
    long_description = open("README.rst").read(),
    license = "MIT",
    url = "http://github.com/pullswitch/django-freeplay",
    packages = ["freeplay"],
    package_data = {"freeplay": ["templates/admin/freeplay/*", "templatetags/*.py"]},
    install_requires = [
        "django-model-utils==1.1.0",
        "django-relatedadminwidget==0.0.2",
        "django-imagekit>=3.0.4"
    ],
    classifiers = [
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Framework :: Django",
    ]
)
